@@ -47,7 +47,7 @@ def get_gaction_from_gtk_action(gtk_action, parent):
     # check if it has signal handler
     callback = gtk_action.get_data(tags.SIGNAL_HANDLER)
     if callback is not None:
-        gaction.set_property('callback', callback)
+        gaction.callback = callback
      
     return gaction
 
@@ -220,6 +220,8 @@ class GActionsView(gtk.ScrolledWindow):
     """
     def __init__(self):
         gtk.ScrolledWindow.__init__(self)
+        self.set_shadow_type(gtk.SHADOW_IN)
+        
         self.project = None
 
         self._model = gtk.TreeStore(object)
