@@ -22,6 +22,7 @@ import xml.dom.minidom
 from xml.sax.saxutils import unescape
 
 from gazpacho.loader import tags
+from gazpacho.loader import compat
 from gazpacho.gaction import GAction
 from gazpacho.l10n import _
 from gazpacho import util
@@ -60,12 +61,14 @@ class WidgetTree(object):
     def _reset(self):
         self._modules = {}
         # put at least the gtk module so existing glade files don't break
-        self._modules['Gtk'] = __import__('gtk')
+        self._modules['gtk'] = __import__('gtk')
         
         self._widgets = {}
         self._signals = []
         self._actions = {}
         self._uimanager = None
+
+        self._deprecated_widgets = []
         
     def load_from_file(self, path):
         self._reset()
@@ -117,13 +120,21 @@ class WidgetTree(object):
                         widget.set_transient_for(self._app.window)
                     widget.show_all()
 
+        # check if we have any deprecated widget
+        if self._deprecated_widgets:
+            if self._app is not None:
+                dialog = compat.DeprecatedWidgetsDialog(self._app.window,
+                                                        self._deprecated_widgets)
+                dialog.run()
+                dialog.destroy()
+                
     def _add_requirement(self, xml_node):
         assert xml_node.tagName == tags.XML_TAG_REQUIRES
 
-        prefix = xml_node.getAttribute(tags.XML_TAG_PREFIX)
-        if not prefix in self._modules.keys():
-            module = xml_node.getAttribute(tags.XML_TAG_LIB)            
-            self._modules[prefix] = __import__(module)
+        module = xml_node.getAttribute(tags.XML_TAG_LIB)
+        if not module in self._modules.keys():
+            module = xml_node.getAttribute(tags.XML_TAG_LIB)
+            self._modules[module.lower()] = __import__(module)
 
     def _build_instance(self, class_name):
         """Look for class_name into the read modules and create an
@@ -132,7 +143,8 @@ class WidgetTree(object):
         widget = None
         found = False
         for prefix, module in self._modules.items():
-            if prefix == class_name[:len(prefix)]:
+            class_prefix = class_name[:len(prefix)]
+            if prefix == class_prefix.lower():
                 real_name = class_name[len(prefix):]
                 try:
                     widget = getattr(module, real_name)()
@@ -166,10 +178,11 @@ class WidgetTree(object):
             obj.set_property('name', id)
             self._fill_object_from_node(obj, xml_node)
 
-        try:
-            self._widgets[obj.get_property('name')] = obj
-        except TypeError:
-            pass # not all the objects have a name
+        if obj is not None:
+            try:
+                self._widgets[obj.get_property('name')] = obj
+            except TypeError:
+                pass # not all the objects have a name
 
         return obj
 
@@ -238,18 +251,7 @@ class WidgetTree(object):
         # children
         for child_node in util.xml_filter_nodes(xml_node.childNodes,
                                                 xml_node.ELEMENT_NODE):
-            if child_node.tagName == tags.XML_TAG_PROPERTY:
-                name = child_node.getAttribute(tags.XML_TAG_NAME)
-                if name == 'uifile':
-                    value = util.xml_get_text_from_node(xml_node)
-                    self._uimanager.add_ui_from_file(value)
-                elif name == 'ui':
-                    # the data is inside a CDATA section
-                    value = child_node.childNodes[0]
-                    self._uimanager.add_ui_from_string(value.data)
-                else:
-                    print 'Unknown property for UIManager: %s' % name
-            elif child_node.tagName == tags.XML_TAG_CHILD:
+            if child_node.tagName == tags.XML_TAG_CHILD:
                 child = self._build_child_object(self._uimanager, child_node)
                 self._uimanager.insert_action_group(child, 0)
 
@@ -273,10 +275,9 @@ class WidgetTree(object):
                 # register this action with our actions dictionary
                 self._actions[action.get_name()] = action
 
-                callback = action.get_data('callback')
+                callback = action.get_data(tags.SIGNAL_HANDLER)
                 if callback is not None:
                     # register the signal handler
-                    action.set_data(tags.SIGNAL_HANDLER, callback)
                     self._signals.append([action, 'activate', callback, None])
                     
         return action_group
@@ -298,18 +299,36 @@ class WidgetTree(object):
                 action.set_data('accelerator', props['accelerator'])
         if 'callback' in props.keys():
             if props['callback']:
-                action.set_data('callback', props['callback'])
+                action.set_data(tags.SIGNAL_HANDLER, props['callback'])
             
         return action
     
     def _build_menu_bar_or_toolbar(self, xml_node):
         id = xml_node.getAttribute(tags.XML_TAG_ID)
-        # we don't instantiate the widget but get it from the uimanager
-        obj = self._uimanager.get_widget('ui/%s' % id)
-        obj.set_property('name', id)
-        self._fill_object_from_node(obj, xml_node)        
-        return obj
 
+        if self._uimanager is None:
+            self._uimanager = gtk.UIManager()
+        
+        # look for the 'ui' property
+        ui_prop_node = util.xml_find_node(xml_node, tags.XML_TAG_PROPERTY,
+                                          name='ui')
+        if ui_prop_node is not None:
+            # the data is inside a CDATA section
+            value = ui_prop_node.childNodes[0]
+            merge_id = self._uimanager.add_ui_from_string(value.data)
+            obj = self._uimanager.get_widget('ui/%s' % id)
+            obj.set_data('merge_id', merge_id)
+            xml_node.removeChild(ui_prop_node)
+            self._fill_object_from_node(obj, xml_node)
+        else:
+            # no ui property, it's probably a Glade-2 menubar/toolbar
+            obj = compat.build_deprecated_widget(xml_node, self._uimanager,
+                                                 self._signals)
+            self._deprecated_widgets.append(obj)
+            
+        obj.set_property('name', id)
+        return obj
+    
     def _read_property(self, obj, xml_node):
         name, value = util.xml_read_raw_property(xml_node)
         for pspec in gobject.list_properties(obj):
