@@ -174,7 +174,7 @@ class Widget(gobject.GObject):
             try:
                 prop.value = self.widget.get_property(p_class.id)
             except TypeError:
-                pass
+                continue
             for p in self._properties:
                 if p.klass.id == p_class.id:
                     p.value = prop.value
@@ -641,6 +641,12 @@ class Widget(gobject.GObject):
                             visible_node = child
                         else:
                             node.appendChild(child)
+        # we'll make an exception with the toolbar/menubar ui property
+        if isinstance(self.widget, (gtk.Toolbar, gtk.MenuBar)):
+            prop = self.get_glade_property('ui')
+            child = prop.write_in_cdata(document)
+            node.appendChild(child)
+                      
         if visible_node is not None:
             node.appendChild(visible_node)
 
